@@ -1,3 +1,4 @@
+{-# LANGUAGE CPP #-}
 {- |
   Module      :  Language.Haskell.Meta.Parse
   Copyright   :  (c) Matt Morrow 2008
@@ -61,7 +62,7 @@ myDefaultParseMode = ParseMode
   ,extensions = map EnableExtension myDefaultExtensions
   ,ignoreLinePragmas = False
   ,ignoreLanguagePragmas = False
-  ,fixities = Just baseFixities}
+  ,fixities = Nothing}
 
 myDefaultExtensions :: [KnownExtension]
 myDefaultExtensions = [PostfixOperators
