@@ -171,34 +171,21 @@ instance ToLit Hs.Literal where
 instance ToPat Hs.Pat where
   toPat (Hs.PVar n)
     = VarP (toName n)
-  toPat (Hs.PLit l)
+  toPat (Hs.PLit Hs.Signless l)
     = LitP (toLit l)
-  toPat (Hs.PNeg (Hs.PLit l)) = LitP $ case toLit l of
+  toPat (Hs.PLit Hs.Negative l) = LitP $ case toLit l of
     IntegerL z -> IntegerL (negate z)
     RationalL q -> RationalL (negate q)
     IntPrimL z' -> IntPrimL (negate z')
     FloatPrimL r' -> FloatPrimL (negate r')
     DoublePrimL r'' -> DoublePrimL (negate r'')
     _ -> nonsense "toPat" "negating wrong kind of literal" l
-  toPat (Hs.PNeg p) = nonsense "toPat" "negating non-literal" p
-#if MIN_VERSION_template_haskell(2,7,0)
   toPat (Hs.PInfixApp p n q) = UInfixP (toPat p) (toName n) (toPat q)    
-#else
-  toPat (Hs.PInfixApp p n q)= InfixP (toPat p) (toName n) (toPat q)
-#endif
   toPat (Hs.PApp n ps) = ConP (toName n) (fmap toPat ps)
   toPat (Hs.PTuple Hs.Boxed ps) = TupP (fmap toPat ps)
-#if MIN_VERSION_template_haskell(2,6,0)
   toPat (Hs.PTuple Hs.Unboxed ps) = UnboxedTupP (fmap toPat ps)
-#else
-  toPat p@(Hs.PTuple Hs.Unboxed _) = noTHyet "toPat" "2.6.0" p
-#endif
   toPat (Hs.PList ps) = ListP (fmap toPat ps)
-#if MIN_VERSION_template_haskell(2,7,0)  
   toPat (Hs.PParen p) = ParensP (toPat p)  
-#else
-  toPat (Hs.PParen p) = toPat p
-#endif
   toPat (Hs.PRec n pfs) = let toFieldPat (Hs.PFieldPat n p) = (toName n, toPat p)
                           in RecP (toName n) (fmap toFieldPat pfs)
   toPat (Hs.PAsPat n p) = AsP (toName n) (toPat p)
@@ -232,63 +219,57 @@ toFieldExp (Hs.FieldUpdate n e) = (toName n, toExp e)
 
 instance ToExp Hs.Exp where
   toExp (Hs.Var n)                 = VarE (toName n)
+  toExp e@Hs.IPVar{}               = noTH "toExp" e
   toExp (Hs.Con n)                 = ConE (toName n)
   toExp (Hs.Lit l)                 = LitE (toLit l)
-#if MIN_VERSION_template_haskell(2,7,0)
   toExp (Hs.InfixApp e o f)        = UInfixE (toExp e) (toExp o) (toExp f)
-#else
-  toExp (Hs.InfixApp e o f)        = InfixE (Just . toExp $ e) (toExp o) (Just . toExp $ f)
-#endif
-  toExp (Hs.LeftSection e o)       = InfixE (Just . toExp $ e) (toExp o) Nothing
-  toExp (Hs.RightSection o f)      = InfixE Nothing (toExp o) (Just . toExp $ f)
   toExp (Hs.App e f)               = AppE (toExp e) (toExp f)
   toExp (Hs.NegApp e)              = AppE (VarE 'negate) (toExp e)
   toExp (Hs.Lambda _ ps e)         = LamE (fmap toPat ps) (toExp e)
   toExp (Hs.Let bs e)              = LetE (hsBindsToDecs bs) (toExp e)
   toExp (Hs.If a b c)              = CondE (toExp a) (toExp b) (toExp c)
+#if MIN_VERSION_template_haskell(2,8,0)
+  toExp (Hs.MultiIf ifs)           = MultiIfE (map toGuard ifs)
+#else
+  toExp e@Hs.MultiIf{}             = noTHyet "toExp" "2.8.0" e
+#endif
+  toExp (Hs.Case e alts)           = CaseE (toExp e) (map toMatch alts)
   toExp (Hs.Do ss)                 = DoE (map toStmt ss)
   toExp e@(Hs.MDo _)               = noTH "toExp" e
   toExp (Hs.Tuple Hs.Boxed xs)     = TupE (fmap toExp xs)
-#if MIN_VERSION_template_haskell(2,6,0)
   toExp (Hs.Tuple Hs.Unboxed xs)   = UnboxedTupE (fmap toExp xs)
-#else
-  toExp e@(Hs.Tuple Hs.Unboxed _)  = noTHyet "toExp" "2.6.0" e
-#endif
+  toExp e@Hs.TupleSection{}        = noTH "toExp" e
   toExp (Hs.List xs)               = ListE (fmap toExp xs)
-#if MIN_VERSION_template_haskell(2,7,0)
   toExp (Hs.Paren e)               = ParensE (toExp e)
-#else
-  toExp (Hs.Paren e)               = toExp e
-#endif
+  toExp (Hs.LeftSection e o)       = InfixE (Just . toExp $ e) (toExp o) Nothing
+  toExp (Hs.RightSection o f)      = InfixE Nothing (toExp o) (Just . toExp $ f)
   toExp (Hs.RecConstr n xs)        = RecConE (toName n) (fmap toFieldExp xs)
   toExp (Hs.RecUpdate e xs)        = RecUpdE (toExp e) (fmap toFieldExp xs)
   toExp (Hs.EnumFrom e)            = ArithSeqE $ FromR (toExp e)
   toExp (Hs.EnumFromTo e f)        = ArithSeqE $ FromToR (toExp e) (toExp f)
   toExp (Hs.EnumFromThen e f)      = ArithSeqE $ FromThenR (toExp e) (toExp f)
   toExp (Hs.EnumFromThenTo e f g)  = ArithSeqE $ FromThenToR (toExp e) (toExp f) (toExp g)
-  toExp (Hs.ExpTypeSig _ e t)      = SigE (toExp e) (toType t)
   toExp (Hs.ListComp e ss)         = CompE $ map convert ss ++ [NoBindS (toExp e)]
    where
     convert (Hs.QualStmt st) = toStmt st
     convert s = noTH "toExp ListComp" s
-  toExp (Hs.Case e alts) = CaseE (toExp e) (map toMatch alts)
+  toExp (Hs.ExpTypeSig _ e t)      = SigE (toExp e) (toType t)
   toExp e = todo "toExp" e
 
 
 toMatch :: Hs.Alt -> Match
-toMatch (Hs.Alt _ p galts ds) = Match (toPat p) (toBody galts) (toDecs ds)
+toMatch (Hs.Alt _ p rhs ds) = Match (toPat p) (toBody rhs) (toDecs ds)
 
-toBody :: Hs.GuardedAlts -> Body
-toBody (Hs.UnGuardedAlt  e) = NormalB $ toExp e
-toBody (Hs.GuardedAlts alts) = GuardedB $ do
-  Hs.GuardedAlt _ stmts e <- alts
-  let
+toBody :: Hs.Rhs -> Body
+toBody (Hs.UnGuardedRhs e) = NormalB $ toExp e
+toBody (Hs.GuardedRhss rhss) = GuardedB $ map toGuard rhss
+
+toGuard (Hs.GuardedRhs _ stmts e) = (g, toExp e)
+  where
     g = case map toStmt stmts of
       [NoBindS x] -> NormalG x
       xs -> PatG xs
-  return (g, toExp e)
 
-toGuard (Hs.GuardedAlt _ ([Hs.Qualifier e1]) e2) = (NormalG $ toExp e1,toExp e2)
 
 -----------------------------------------------------------------------------
 
@@ -348,11 +329,7 @@ instance ToType Hs.Type where
    where
     tuple = case b of
       Hs.Boxed -> TupleT
-#if MIN_VERSION_template_haskell(2,6,0)
       Hs.Unboxed -> UnboxedTupleT
-#else
-      Hs.Unboxed -> noTHyet "toType TyTuple" "2.6.0" (Hs.TyTuple b ts)
-#endif
   toType (Hs.TyApp a b) = AppT (toType a) (toType b)
   toType (Hs.TyVar n) = VarT (toName n)
   toType (Hs.TyCon qn) = ConT (toName qn)
@@ -360,8 +337,17 @@ instance ToType Hs.Type where
   -- XXX: need to wrap the name in parens!
   toType (Hs.TyInfix a o b) = AppT (AppT (ConT (toName o)) (toType a)) (toType b)
   toType (Hs.TyKind t k) = SigT (toType t) (toKind k)
+  toType t@Hs.TyBang{} =
+    nonsense "toType" "type cannot have strictness annotations in this context" t
 
 
+toStrictType :: Hs.Type -> StrictType
+toStrictType t@(Hs.TyBang _ Hs.TyBang{}) =
+  nonsense "toStrictType" "double strictness annotation" t
+toStrictType (Hs.TyBang Hs.BangedTy t) = (IsStrict, toType t)
+toStrictType (Hs.TyBang Hs.UnpackedTy t) = (Unpacked, toType t)
+toStrictType t = (NotStrict, toType t)
+
 
 
 (.->.) :: Type -> Type -> Type
@@ -370,9 +356,15 @@ a .->. b = AppT (AppT ArrowT a) b
 toCxt :: Hs.Context -> Cxt
 toCxt = fmap toPred
  where
+#if MIN_VERSION_template_haskell(2,10,0)
+  toPred (Hs.ClassA n ts) = foldl' AppT (ConT (toName n)) (fmap toType ts)
+  toPred (Hs.InfixA t1 n t2) = foldl' AppT (ConT (toName n)) (fmap toType [t1,t2])
+  toPred (Hs.EqualP t1 t2) = foldl' AppT EqualityT (fmap toType [t1,t2])
+#else
   toPred (Hs.ClassA n ts) = ClassP (toName n) (fmap toType ts)
   toPred (Hs.InfixA t1 n t2) = ClassP (toName n) (fmap toType [t1, t2])
   toPred (Hs.EqualP t1 t2) = EqualP (toType t1) (toType t2)
+#endif
   toPred a@Hs.IParam{} = noTH "toCxt" a
 
 foldAppT :: Type -> [Type] -> Type
@@ -386,6 +378,7 @@ instance ToStmt Hs.Stmt where
   toStmt (Hs.Generator _ p e)  = BindS (toPat p) (toExp e)
   toStmt (Hs.Qualifier e)      = NoBindS (toExp e)
   toStmt a@(Hs.LetStmt bnds)   = LetS (hsBindsToDecs bnds)
+  toStmt s@Hs.RecStmt{}        = noTH "toStmt" s
 
 
 -----------------------------------------------------------------------------
@@ -397,11 +390,6 @@ hsBindsToDecs (Hs.BDecls ds) = fmap toDec ds
 hsBindsToDecs a@Hs.IPBinds{} = noTH "hsBindsToDecs" a
 
 
-hsBangTypeToStrictType :: Hs.BangType -> (Strict, Type)
-hsBangTypeToStrictType (Hs.BangedTy t)   = (IsStrict, toType t)
-hsBangTypeToStrictType (Hs.UnBangedTy t) = (NotStrict, toType t)
-
-
 instance ToDec Hs.Decl where
   toDec (Hs.TypeDecl _ n ns t)
     = TySynD (toName n) (fmap toTyVar ns) (toType t)
@@ -456,13 +444,17 @@ instance ToDec Hs.Decl where
     = FamilyD DataFam (toName n) (fmap toTyVar ns) (fmap toKind k)
 
   toDec a@(Hs.FunBind mtchs)                           = hsMatchesToFunD mtchs
-  toDec (Hs.PatBind _ p tM rhs bnds)                   = ValD ((maybe id
-                                                                      (flip SigP . toType)
-                                                                      tM) (toPat p))
+  toDec (Hs.PatBind _ p rhs bnds)                      = ValD (toPat p)
                                                               (hsRhsToBody rhs)
                                                               (hsBindsToDecs bnds)
 
-  toDec (Hs.InstDecl _ cxt qname ts ids) = InstanceD 
+  toDec i@(Hs.InstDecl _ (Just overlap) _ _ _ _ _) =
+    noTH "toDec" (overlap, i)
+
+  -- the 'vars' bit seems to be for: instance forall a. C (T a) where ...
+  -- TH's own parser seems to flat-out ignore them, and honestly I can't see
+  -- that it's obviously wrong to do so.
+  toDec (Hs.InstDecl _ Nothing _vars cxt qname ts ids) = InstanceD 
     (toCxt cxt) 
     (foldl AppT (ConT (toName qname)) (map toType ts))
     (toDecs ids)
@@ -501,19 +493,13 @@ qualConDeclToCon (Hs.QualConDecl _ ns cxt cdecl) = ForallC (fmap toTyVar ns)
                                                     (conDeclToCon cdecl)
 conDeclToCon :: Hs.ConDecl -> Con
 conDeclToCon (Hs.ConDecl n tys)
-  = NormalC (toName n) (fmap bangToStrictType tys)
-conDeclToCon (Hs.RecDecl n lbls)
-  = RecC (toName n) (concatMap (uncurry bangToVarStrictTypes) lbls)
-
-
-bangToVarStrictTypes :: [Hs.Name] -> Hs.BangType -> [VarStrictType]
-bangToVarStrictTypes ns t = let (a,b) = bangToStrictType t
-                            in fmap (\n->(toName n,a,b)) ns
-
-bangToStrictType :: Hs.BangType -> StrictType
-bangToStrictType (Hs.BangedTy   t) = (IsStrict, toType t)
-bangToStrictType (Hs.UnBangedTy t) = (NotStrict, toType t)
-bangToStrictType (Hs.UnpackedTy t) = (IsStrict, toType t)
+  = NormalC (toName n) (map toStrictType tys)
+conDeclToCon (Hs.RecDecl n fieldDecls)
+  = RecC (toName n) (concatMap convField fieldDecls)
+  where
+    convField (fields, t) =
+      let (strict, ty) = toStrictType t
+      in map (\field -> (toName field, strict, ty)) fields
 
 
 hsMatchesToFunD :: [Hs.Match] -> Dec
